@@ -1,5 +1,11 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import absolute_import, print_function, division
+try:
+    from future_builtins import zip, map  # noqa: F401; Use Python 3 "lazy" zip, map
+except ImportError:
+    pass
+
 import binascii
 import codecs
 import copy
@@ -15,21 +21,22 @@ try:
 except ImportError:
     chacha20poly1305		= None
 
-from dns.exception import DNSException
-from .verification import (
+from dns.exception	import DNSException
+from .verification	import (
     License, LicenseSigned, LicenseIncompatibility, Timespan, Agent,
     KeypairPlaintext, KeypairEncrypted, machine_UUIDv4,
     domainkey, domainkey_service, overlap_intersect,
-    into_b64, into_hex, into_str, into_str_UTC, into_JSON, into_keys,
+    into_b64, into_hex, into_str, into_str_UTC, into_JSON, into_keys, into_bytes,
     into_Timestamp, into_Duration,
     authoring, issue, verify, load, load_keys, check, authorized,
+    DKIM_pubkey, DKIMError,
 )
-from .. import ed25519
+from ..			import ed25519
 
-from ..misc import (
+from ..misc 		import (
     deduce_name, Timestamp
 )
-from .defaults import LICEXTENSION
+from .defaults		import LICEXTENSION
 
 log				= logging.getLogger( "verification_test" )
 
@@ -58,6 +65,14 @@ def test_Agent():
 }"""
 
 
+def test_DKIM():
+    assert DKIM_pubkey( 'v=DKIM1; k=ed25519; p=25lf4lFp0UHKubu6krqgH58uHs599MsqwFGQ83/MH50=' ) \
+        == into_bytes( '25lf4lFp0UHKubu6krqgH58uHs599MsqwFGQ83/MH50=', ('base64',) )
+    with pytest.raises( DKIMError ) as exc_info:
+        DKIM_pubkey( 'v=DKIM2; k=ed25519; p=25lf4lFp0UHKubu6krqgH58uHs599MsqwFGQ83/MH50=' )
+    assert "Failed to find 'DKIM1' record" in str( exc_info )
+
+
 def test_License_domainkey():
     """Ensure we can handle arbitrary UTF-8 domains, and compute the proper DKIM1 RR path"""
     assert domainkey_service( u"π" ) == 'xn--1xa'
@@ -77,6 +92,16 @@ def test_License_domainkey():
     assert path == 'xn--raspberry--1qh.crypto-licensing._domainkey.xn--8dbaco.email.'
     assert dkim_rr == 'v=DKIM1; k=ed25519; p=25lf4lFp0UHKubu6krqgH58uHs599MsqwFGQ83/MH50='
 
+    # "Лайка.ru" wants to run a crypto-licensing server, selling  "Raspberry π"
+    # http://crypto-licensing.xn--80aa0aec.ru/ (crypto-licensing.Лайка.ru)
+    path, dkim_rr = domainkey( u"Raspberry π", u"Лайка.ru", pubkey=author_keypair )
+    assert path == 'xn--raspberry--1qh.crypto-licensing._domainkey.xn--80aa0aec.ru.'
+    # And make certain round-tripping punycoded domain names doesn't break
+    path, dkim_rr = domainkey( "xn--raspberry--1qh", "xn--80aa0aec.ru", pubkey=author_keypair )
+    assert path == 'xn--raspberry--1qh.crypto-licensing._domainkey.xn--80aa0aec.ru.'
+    path, dkim_rr = domainkey( u"xn--raspberry--1qh", u"xn--80aa0aec.ru", pubkey=author_keypair )
+    assert path == 'xn--raspberry--1qh.crypto-licensing._domainkey.xn--80aa0aec.ru.'
+
 
 def test_License_overlap():
     """A License can only issued while all the sub-Licenses are valid.  The start/length should "close"
